package application;

public class Utils {

	// real-world display size and user viewing distance
	static private double distToDisp; // meter
	static private double dispWidth;  // meter
	static private double dispHeight; // meter
	
	// screen resolutions 
	static private double scrWidth;  // pixels
	static private double scrHeight; // pixels

	static private double meterPerPixelX; // meter/pixel
	static private double meterPerPixelY; // meter/pixel
	static private double pixelsPerMeterX;// pixel/meter 
	static private double pixelsPerMeterY;// pixel/meter  
	
	// temporary values
	static private double tempX;
	static private double tempY;
	
	// initialize Utils class
	public static void init(double distToDisplay, 
							double displayWidth, double displayHeight,
							double screenWidth, double screenHeight) {
		distToDisp = distToDisplay;
		dispWidth  = displayWidth;
		dispHeight = displayHeight;
		scrWidth   = screenWidth;
		scrHeight  = screenHeight;
		meterPerPixelX = dispWidth/scrWidth; 
		meterPerPixelY = dispHeight/scrHeight;
		pixelsPerMeterX = scrWidth/dispWidth; 
		pixelsPerMeterY = scrHeight/dispHeight; 
	}
	
	// compute real-world coordinate from onscreen pixel coordinate
	public static double getRealWorldCoordX(double pixX, double realZ) {
		tempX = (pixX-scrWidth/2)*meterPerPixelX;		
		return (realZ/distToDisp)*tempX;	
	}
	public static double getRealWorldCoordY(double pixY, double realZ) {
		tempY = (pixY-scrHeight/2)*meterPerPixelY;		
		return (realZ/distToDisp)*tempY;	
	}

	// compute onscreen pixel coordinate from real-world coordinates
	public static double getOnscreenCoordX(double realX, double realZ) {
		tempX = ((distToDisp*realX)/realZ)*pixelsPerMeterX;			
		return scrWidth/2+tempX;		
	}	
	public static double getOnscreenCoordY(double realY, double realZ) {
		tempY = ((distToDisp*realY)/realZ)*pixelsPerMeterY;			
		return scrHeight/2+tempY;		
	}	

	// getter for display FoV in radians
	public static double getDisplayHorizontalFoV() {
		return Math.atan2(distToDisp, dispWidth/2.0)*2.0;
	}
	public static double getDisplayVerticalFoV() {
		return Math.atan2(distToDisp, dispHeight/2.0)*2.0;
	}

	// Main program for launcher
    public static void main(String[] args) {
        double SCREEN_WIDTH 	= 1920; // pixels
        double SCREEN_HEIGHT 	= 1080; // pixels
        
        double DISPLAY_WIDTH 	= 0.51; // meter
        double DISPLAY_HEIGHT 	= 0.29; // meter
        double DISTANCE_TO_DISPLAY = 0.6; // meter
        
    	// initialize Utility function
    	Utils.init(DISTANCE_TO_DISPLAY, DISPLAY_WIDTH, DISPLAY_HEIGHT, SCREEN_WIDTH, SCREEN_HEIGHT);
    	System.out.println(Utils.getRealWorldCoordX(1920, 10));
    	System.out.println(Utils.getOnscreenCoordX(4.25, 10));
    	System.out.println(Utils.getRealWorldCoordY(1080, 10));
    	System.out.println(Utils.getOnscreenCoordY(2.4166666666666665, 10));
    	
    	System.out.println(Utils.getRealWorldCoordX(0, 10));
    	System.out.println(Utils.getOnscreenCoordX(-4.25, 10));
    	System.out.println(Utils.getRealWorldCoordY(0, 10));
    	System.out.println(Utils.getOnscreenCoordY(-2.4166666666666665, 10));
    }
    
}
